	LIST p = pic16f84
;**************************************************************************
;***      MACRO NAME:  DELAY
;***        Function:  Delays for specified length of time.
;***          Range:   0 to 768 instruction cycles.
;*** Registers Used:   DLYTMR  Counts machine cycles
;***                   TMP1    Used to store and recall W
;*** Pass Paramters:   DTIME set delay.  Must be < 256*3 (768)
;***
;*** Syntax:           DELAY XXX ;where XXX is a number from 0 to 768
;***
;*** USES REGISTERS - TMP1 DLYTMR
;***
;*** Features:
;***    Uses no stack space.
;***    Creates minimum amount of code possible.
;***    Created code provides exact number of instruction cycle delays.
;***    Does not destroy contents of W, FSR, or PCLATH
;***
;*** Revised 9/26/94   DELAY is now non-destructive of the contents of the
;***                   W register, either by not changing it, or by saving
;***                   and restoring after the delay.
;*** Revised 9/29/94   Reworked delay macro to use formulae at compile time,
;***                   restoring exact delay times for long delays.
;*****************************************************************************
;*****************************************************************************
;*****************************************************************************
;***                        WARNING!                                       ***
;***   The following set of instructions has been carefully crafted to     ***
;***   occur in a specific number of machine cycles.  Altering these       ***
;***   instructions may result in different timing which will adversly     ***
;***   affect the overall timing of the main program.                      ***
;***                                                                       ***
;*****************************************************************************
;*****************************************************************************
DELAY  MACRO  DTIME
        LOCAL  DLOOP
;============================================================================
   IF DTIME==0
        EXITM               ; If no delay time, do nothing.
   ENDIF
;
   IF DTIME==1
   NOP         ;DELAY 1
   EXITM
   ENDIF
;
   IF DTIME==2
   GOTO   $+1  ;DELAY 2 (2-cycle NOP)
   EXITM
   ENDIF
;
   IF DTIME==3
   NOP         ;DELAY
   GOTO   $+1  ;  3
   EXITM
   ENDIF
;
   IF DTIME==4
   GOTO   $+1  ;DELAY
   GOTO   $+1  ;  4
   EXITM
   ENDIF
;
   IF DTIME==5
   NOP         ;\ DELAY
   GOTO   $+1  ;   5
   GOTO   $+1  ;/ CYCLES
   EXITM
   ENDIF
;
   IF DTIME==6
   GOTO   $+1  ;\ DELAY
   GOTO   $+1  ;   6
   GOTO   $+1  ;/ CYCLES
   EXITM
   ENDIF
;
   IF DTIME==7
   NOP         ;\ DELAY
   GOTO   $+1  ; \  7
   GOTO   $+1  ; /
   GOTO   $+1  ;/ CYCLES
   EXITM
   ENDIF
;
   IF DTIME==8
   GOTO   $+1  ;\ DELAY
   GOTO   $+1  ; \  8
   GOTO   $+1  ; /
   GOTO   $+1  ;/ CYCLES
   EXITM
   ENDIF
;
   IF DTIME==9
   NOP         ;\  DELAY
   GOTO   $+1  ; \   9
   GOTO   $+1  ;
   GOTO   $+1  ; /
   GOTO   $+1  ;/  CYCLES
   EXITM
   ENDIF
;The following routine Takes a minimum of 4 (outside loop) + 2 (single
;cycle of loop) = 6 cycles.  It also takes 6 bytes of code space, so
;2-cycle NOP approach is preferred up to 9 cycles (4 2-cycle & NOP = 5 bytes).
;
;To calculate the literal loop counter, the following formula is used.
;First subtract the 4 cycles taken by instructions outside the loop.
;Next, devide the remaining time by 3 to discover how many times to
;execute the loop.
;Finally, conditionally insert none, a 1-cycle, or a 2-cycle NOP to handle
;any time remaining due to a remainder when dividing by 3.
; DTIME                          < The time to be delayed.
; (DTIME-4)                      < The time to be used by loop + nops
; (DTIME-3)/3 set = LPTIME       < The number of loops required.
; ((LPTIME * 3) - 1)             < The amount of time actually used by the
loop
; ((LPTIME * 3) + 3)             < The amount of time taken in total so far.
; (DTIME - ((LPTIME*3)+3))       < Remainder 0,1, or 2 (determines nops rq'd)
   IF DTIME9
LPTIME SET ((DTIME-3)/3)        ;
   MOVWF  TMP1                   ;Save W
   MOVLW  LPTIME                 ;
   MOVWF  DLYTMR                 ;
DLOOP  DECFSZ DLYTMR,f          ;
   GOTO   DLOOP                  ;
   ENDIF
   IF (DTIME - ((LPTIME*3)+3) == 1);Check for 1 remaining cycle.
   NOP                           ;
   ENDIF
   IF (DTIME - ((LPTIME*3)+3) == 2);Check for 2 remaining cycles.
   GOTO   $+1                    ;
   ENDIF
   MOVF   TMP1,W                 ;Restore W
   ENDM
   
;*************************************
;***   END OF TIME CRITICAL CODE   ***
;*************************************
	END
